import datetime

import numpy as np
import pandas as pd


def calculate_business_days_delta(date_1: datetime.datetime, date_2: datetime.datetime):
    if date_1 is None or date_2 is None or pd.isna(date_1) or pd.isna(date_2):
        return None
    return max(0, np.busday_count(date_1.strftime("%Y-%m-%d"), date_2.strftime("%Y-%m-%d")))



import numpy as np
import pandas as pd

from ovretl.shipment_orchestration_utils.so_workflows_weights import so_workflows_weights


def test_so_workflows_weights():

    wf_df = pd.DataFrame(data={"id": ["1", "2", "3"], "name": ["wf1", "wf2", "wf3"],})
    st_df = pd.DataFrame(data={"id": ["1", "2", "3", "4"], "so_workflow_id": ["1", "2", "3", "3"],})
    sa_assoc_df = pd.DataFrame(
        data={
            "id": ["1", "2", "3", "4", "5"],
            "so_state_id": ["1", "2", "2", "3", "4"],
            "so_action_id": ["1", "2", "3", "4", "5"],
        }
    )
    ac_df = pd.DataFrame(data={"id": ["1", "2", "3", "4", "5"], "weight": [1, 1, 1, 1, 1],})
    result_should_be = pd.DataFrame(
        data={"id": ["1", "2", "3"], "name": ["wf1", "wf2", "wf3"], "total_workflow_weight": [1, 2, 2]}
    )

    result = so_workflows_weights(wf_df, st_df, sa_assoc_df, ac_df)
    pd.testing.assert_frame_equal(result, result_should_be)



import pandas as pd
from ovretl.containers_utils.calculate_single_container_teus import calculate_single_container_teus


def aggregate_containers(containers: pd.DataFrame):
    aggregated_containers = {}
    aggregated_containers["teus"] = containers["teus"].sum()
    aggregated_containers["tc"] = len(containers)
    aggregated_containers["hazardous"] = containers["hazardous"].any()
    aggregated_containers["refrigerated"] = (
        containers["container_type"]
        .isin(["twenty_standard_reefer", "forty_standard_reefer", "forty_highcube_reefer"])
        .any()
    )
    return pd.Series(aggregated_containers, index=["teus", "tc", "hazardous", "refrigerated"])


def calculate_entity_containers(containers_df: pd.DataFrame, key: str):
    entity_containers = containers_df[~containers_df[key].isnull()]
    return entity_containers.groupby(key).apply(aggregate_containers).reset_index().drop_duplicates(subset=[key])


def calculate_shipments_propositions_containers(containers_df: pd.DataFrame):
    containers_df = containers_df.apply(calculate_single_container_teus, axis=1)
    containers_df = containers_df.dropna(subset=["teus"])
    propositions_containers = calculate_entity_containers(containers_df=containers_df, key="proposition_id")
    shipments_containers = calculate_entity_containers(containers_df=containers_df, key="shipment_id")
    return pd.concat([propositions_containers, shipments_containers], sort=False)



import boto3

from ast import literal_eval
from botocore.exceptions import ClientError

cluster_secret_map = {
    "dev": "dev-password",
    "prod": "prod-password",
    "analytics": "OVRSEA_ANALYTICS",
}


def fetch_db_credentials(cluster="dev"):
    secret_name = cluster_secret_map[cluster]
    region_name = "eu-west-1"

    session = boto3.session.Session()
    client = session.client(service_name="secretsmanager", region_name=region_name,)
    try:
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)
    except ClientError as e:
        if e.response["Error"]["Code"] == "ResourceNotFoundException":
            print("The requested secret " + secret_name + " was not found")
        elif e.response["Error"]["Code"] == "InvalidRequestException":
            print("The request was invalid due to:", e)
        elif e.response["Error"]["Code"] == "InvalidParameterException":
            print("The request had invalid params:", e)
    else:
        # Secrets Manager decrypts the secret value using the associated KMS CMK
        # Depending on whether the secret was a string or binary, only one of these fields will be populated
        if "SecretString" in get_secret_value_response:
            text_secret_data = get_secret_value_response["SecretString"]
            return literal_eval(text_secret_data)
        else:
            binary_secret_data = get_secret_value_response["SecretBinary"]
            return literal_eval(binary_secret_data.decode("ascii"))



import pandas as pd


def join_tracking_events(tracking_records_df: pd.DataFrame, events_df: pd.DataFrame):
    return pd.merge(
        events_df, tracking_records_df[["id", "shipment_id"]], left_on="tracking_record_id", right_on="id", how="inner",
    )



import pandas as pd
import datetime

from ovretl.tasks_utils.so_tasks_filter import so_tasks_filter


def test_so_tasks_filter():
    now_datetime = datetime.datetime.now()
    now_date = now_datetime.date()
    tasks_df = pd.DataFrame(
        data={
            "id": ["1", "2"],
            "so_id": ["1", None],
            "created_at": now_datetime,
            "updated_at": now_datetime,
            "due_date": now_datetime,
            "done_date": now_datetime,
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "id": ["1"],
            "so_id": ["1"],
            "created_at": now_datetime,
            "updated_at": now_datetime,
            "due_date": now_datetime,
            "done_date": now_datetime,
        }
    )
    result = so_tasks_filter(tasks_df)
    pd.testing.assert_frame_equal(result, result_should_be)



import pandas as pd
import numpy as np

SHIPMENT_STRICT_STATUS = ["booking_request", "awaiting_booking", "booked", "in_progress", "finished", "arrived"]


def sort_and_index_group(group: pd.DataFrame):
    sorted_group = group.sort_values("created_at", ascending=True)
    sorted_group.index = np.arange(1, len(sorted_group) + 1)
    return sorted_group


def add_client_order_index(quotation_df: pd.DataFrame, index_column: str):
    quotation_df = quotation_df.groupby(["client_name"]).apply(lambda g: sort_and_index_group(g))
    quotation_df = quotation_df.drop(columns="client_name").reset_index()
    quotation_df = quotation_df.rename(columns={"level_1": index_column})
    return quotation_df


def add_client_quotation_shipment_index(quotation_df: pd.DataFrame) -> pd.DataFrame:
    quotation_df = add_client_order_index(quotation_df, index_column="quotation_number")
    shipments_df = quotation_df[quotation_df["shipment_status"].isin(SHIPMENT_STRICT_STATUS)].copy()
    shipments_df = add_client_order_index(shipments_df, index_column="shipment_number")

    return pd.merge(quotation_df, shipments_df[["foresea_name", "shipment_number"]], on="foresea_name", how="left",)



from datetime import datetime, date, timedelta


def get_days_from_timedelta(delta: timedelta):
    return delta.days


def set_negative_to_zero(a: float):
    if a < 0:
        return 0
    else:
        return a


def datetime_to_date(dt: datetime) -> date:
    return dt.date()


def string_date_to_datetime(datestr):
    try:
        return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S.%fZ").date()
    except:
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ovretl',
 'ovretl.billings_utils',
 'ovretl.billings_utils.tests',
 'ovretl.containers_utils',
 'ovretl.containers_utils.tests',
 'ovretl.db_utils',
 'ovretl.employees_utils',
 'ovretl.employees_utils.tests',
 'ovretl.kronos_propositions_utils',
 'ovretl.kronos_propositions_utils.tests',
 'ovretl.loads_utils',
 'ovretl.loads_utils.tests',
 'ovretl.performances_utils',
 'ovretl.performances_utils.tests',
 'ovretl.prices_utils',
 'ovretl.prices_utils.features_functions',
 'ovretl.prices_utils.tests',
 'ovretl.shipment_orchestration_utils',
 'ovretl.shipment_orchestration_utils.tests',
 'ovretl.shipments_utils',
 'ovretl.shipments_utils.tests',
 'ovretl.shipowners_utils',
 'ovretl.shipowners_utils.__tests__',
 'ovretl.tasks_utils',
 'ovretl.tasks_utils.tests',
 'ovretl.tracking_utils',
 'ovretl.tracking_utils.tests',
 'ovretl.transit_times',
 'ovretl.transit_times.tests']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.14.48,<2.0.0',
 'botocore>=1.17.48,<2.0.0',
 'cython==0.29.24',
 'pandas==0.23.4',
 'psycopg2-binary>=2.8.5,<3.0.0',
 'snapshottest>=0.5.1,<0.6.0',
 'time-machine==2.1.0']

setup_kwargs = {
    'name': 'ovretl',
    'version': '5.8.3',
    'description': 'Python package for Ovrsea ETL',
    'long_description': '',
    'author': 'nicolas67',
    'author_email': 'nicolas@ovrsea.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1',
}


setup(**setup_kwargs)



import numpy as np
import pandas as pd


def encode_distance(train_df: pd.DataFrame, feature: str):
    train_df.loc[:, feature] = train_df[feature].apply(lambda x: int(x / 100) * 100 if not pd.isna(x) else np.nan)
    # value_counts = train_df[feature].value_counts()
    # mapping = {key: key if value >= group_size_threshold else DEFAULT_ENCODING for key, value in value_counts.items()}
    # train_df.loc[:, feature] = train_df[feature].replace(mapping)


# def encode_feature(train_df: pd.DataFrame, group_size_threshold: int, feature: str):
#     value_counts = train_df[feature].value_counts()
#     mapping = {key: key if value >= group_size_threshold else DEFAULT_ENCODING for key, value in value_counts.items()}
#     train_df.loc[:, feature] = train_df[feature].replace(mapping)


def encode_features(train_df: pd.DataFrame):
    # qualitative_features = [
    #     "pickup_harbor_country",
    #     "pickup_harbor_locode",
    #     "delivery_harbor_country",
    #     "delivery_harbor_locode",
    #     "pickup_shipowner",
    #     "departure_shipowner",
    #     "arrival_shipowner",
    #     "delivery_shipowner",
    #     "freight_shipowner",
    # ]
    distance_features = ["pickup_distance", "delivery_distance"]
    train_df_copy = train_df.copy()
    # for feature in qualitative_features:
    #     encode_feature(train_df_copy, group_size_threshold, feature)
    for feature in distance_features:
        encode_distance(train_df_copy, feature)
    return train_df_copy
